@@ -1,5 +1,5 @@
 #
-# Copyright CEA/DAM/DIF (2007, 2008, 2009)
+# Copyright CEA/DAM/DIF (2007, 2008, 2009, 2010)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Worker.py 323 2010-08-29 21:00:18Z st-cea $
+# $Id: Worker.py 381 2010-10-11 22:19:08Z st-cea $
 
 """
 ClusterShell worker interface.
@@ -48,14 +48,14 @@ class WorkerException(Exception):
 class WorkerError(WorkerException):
     """Generic worker error."""
 
-class WorkerBadArgumentError(WorkerError):
-    """Bad argument in worker error."""
+# DEPRECATED: WorkerBadArgumentError exception is deprecated as of 1.4,
+# use ValueError instead.
+WorkerBadArgumentError = ValueError
 
 class Worker(object):
     """
     Base class Worker.
     """
-
     def __init__(self, handler):
         """
         Initializer. Should be called from derived classes.
@@ -89,6 +89,8 @@ class Worker(object):
         if self.eh:
             self.eh._invoke(ev_type, self)
 
+    # Base getters
+
     def last_read(self):
         """
         Get last read message from event handler.
@@ -103,11 +105,19 @@ class Worker(object):
 
     def did_timeout(self):
         """
-        Return True if this worker aborted due to timeout.
+        Return whether this worker has aborted due to timeout.
         """
         self._task_bound_check()
         return self.task._num_timeout_by_worker(self) > 0
 
+    # Base actions
+
+    def abort(self):
+        """
+        Abort processing any action by this worker.
+        """
+        raise NotImplementedError("Derived classes must implement.")
+
     def flush_buffers(self):
         """
         Flush any messages associated to this worker.
@@ -396,13 +406,11 @@ class WorkerSimple(EngineClient, Worker):
         """
         return EngineClient._readerr(self, size)
 
-    def _close(self, force, timeout):
+    def _close(self, abort, flush, timeout):
         """
-        Close worker. Called by engine after worker has been
-        unregistered. This method should handle all termination types
-        (normal, forced or on timeout).
+        Close client. See EngineClient._close().
         """
-        if not force and self._rbuf:
+        if flush and self._rbuf:
             # We still have some read data available in buffer, but no
             # EOL. Generate a final message before closing.
             self.worker._on_msgline(self._rbuf)
@@ -415,6 +423,7 @@ class WorkerSimple(EngineClient, Worker):
             self.file_error.close()
 
         if timeout:
+            assert abort, "abort flag not set on timeout"
             self._on_timeout()
 
         self._invoke("ev_close")
